/*
 * Name:     CFrame.cpp
 * This is the main program frame. It owns the menus and all child windows.
 * In this application it also owns the document.
 *
 * Class:    CFrame
 * Header:   CFrame.h
 * Created : Feb 20, 2010
 * Author :  Charles B. Owen - Initial distributed version
 * Co-Author: Kenji Aono
 */

#include "wx/prec.h"
#include "CFrame.h"
#include "CDocument.h"
#include "Factory/CDocumentFactory.h"
#include "View/CViewActors.h"
#include "CTimelineDlg.h"
#include <sstream>

using namespace std;

BEGIN_EVENT_TABLE(CFrame, wxFrame)
    EVT_MENU(ID_Exit, CFrame::OnExit)
    EVT_MENU(ID_About, CFrame::OnAbout)
    EVT_MENU(ID_Save, CFrame::OnSave)
    EVT_MENU(ID_Load, CFrame::OnOpen)
    EVT_MENU(ID_Expand, CFrame::OnExpand)
    EVT_MENU(ID_AnimationProperties, CFrame::OnAnimationProperties)
    EVT_MENU(ID_Player, CFrame::OnPlayer)
    EVT_MENU(ID_Move, CFrame::OnMove)
    EVT_MENU(ID_Rotate, CFrame::OnRotate)
    EVT_MENU(ID_Keyframe, CFrame::OnKeyframeSet)
    EVT_MENU(ID_NoKeyframe, CFrame::OnKeyframeClear)
END_EVENT_TABLE()

/*
 * Name :        CFrame::CFrame()
 * Description : Constructor for our frame.
 */
CFrame::CFrame() : wxFrame( NULL, -1, L"The Canadian Experience",
                            wxPoint(100, 100), wxDefaultSize)
{
    // Create our document
    CDocumentFactory factory;
    mDocument = factory.Create();

    // This is the default mode for the toolbar
    mMode = ID_Move;

    //
    // File menu
    //

    wxMenu *menuFile = new wxMenu;
    mViewFile = new wxMenu;
    wxMenu *animFile = new wxMenu;

    wxMenuItem *loadItem = menuFile->Append(ID_Load, L"&Open File...");
    loadItem->SetHelp(L"Loads an animation from a .canim file");
    wxMenuItem *saveItem = menuFile->Append(ID_Save, L"&Save As...");
    saveItem->SetHelp(L"Save the animation as .canim");
    menuFile->Append(ID_About, L"&About");
    menuFile->AppendSeparator();
    wxMenuItem *exitItem = menuFile->Append( ID_Exit, L"E&xit" );
    exitItem->SetHelp(L"Exit the program");

    mExpandItem = mViewFile->AppendCheckItem(ID_Expand, L"&Expand ");
    mExpandItem->SetHelp(L"Display the objects of an actor");

    wxMenuItem *propItem = animFile->Append( ID_AnimationProperties, L"&Properties...");
    propItem->SetHelp(L"Displays a dialog box with properties for animation");
    wxMenuItem *playItem = animFile->Append( ID_Player, L"&Open a player...");
    playItem->SetHelp(L"Opens a new window to play the current animation");

    //
    // Menu Bar
    //

    wxMenuBar *menuBar = new wxMenuBar;
    menuBar->Append( menuFile, L"&File");
    menuBar->Append( mViewFile, L"&View");
    menuBar->Append( animFile, L"&Animation");

    SetMenuBar( menuBar );

    //
    // Create a status bar
    //

    CreateStatusBar();
    SetStatusText(L"");

    //
    // Create a toolbar
    //

    CreateToolBar(wxHORIZONTAL, ID_ToolBar);
    mToolBar = GetToolBar();

    mToolBar->SetMargins(0, 0);
    AddToToolbar(mToolBar, true, L"images/tb_move.png", ID_Move, L"Move");
    AddToToolbar(mToolBar, true, L"images/tb_rotate.png", ID_Rotate, L"Rotate");
    AddToToolbar(mToolBar, false, L"images/tb_keyframe.png", ID_Keyframe, L"Set Keyframe");
    AddToToolbar(mToolBar, false, L"images/tb_nokeyframe.png", ID_NoKeyframe, L"Clear Keyframe");

    //
    // Create the child windows
    //

    mViewTimeline = new CViewTimeline(this);
    mViewEdit = new CViewEdit(this);
    mViewActors = new CViewActors(this);

    //
    // Create the sizers
    //

    // A vertical sizer so we an put two windows on top of each other
    wxBoxSizer *vertSizer = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *horzSizer = new wxBoxSizer(wxHORIZONTAL);

    horzSizer->Add(mViewActors, 0, wxEXPAND | wxALL, 0);   // Add actors view as first in horzSizer
    horzSizer->Add(mViewEdit, 1, wxEXPAND | wxALL, 0);     // Add edit view as second in horzSizer

    vertSizer->Add(horzSizer, 1, wxEXPAND | wxALL);        // horzSizer is first (top) in vertSizer
    vertSizer->Add(mViewTimeline, 0, wxEXPAND | wxALL, 0); // Add timeline view as second in vertSizer

    SetSizer(vertSizer);
    vertSizer->Fit(this);
    mViewEdit->SetMinSize(wxSize(100, 100));

    SetInterface();
}

/*
 * Name :        CFrame::~CFrame()
 * Description : CFrame destructor. Destroys anything this class owns.
 */
CFrame::~CFrame()
{
	// Cleaning up
	delete mDocument;
}



/*
 * Name :        CFrame::AddToToolbar()
 * Description : Add a bitmap to the toolbar.
 * Parameters :  toolBar - the toolbar we are adding to.
 *               checked - true if we use a checked item (stays pressed).
 *               id - An id for this toolbar button
 *               name - Associated tool name for tooltip.
 * Returns :     None
 */
void CFrame::AddToToolbar(wxToolBar *toolBar, bool checked, const std::wstring &file, int id, const std::wstring &name)
{
    // It is okay to create this as a local variable, because the
    // system actually copies the bitmap to the toolbar, so it's not
    // referred to later.
    wxBitmap bitmap(file.c_str());
    if(checked)
    {
        toolBar->AddCheckTool(id, L"", bitmap, wxNullBitmap, name.c_str());
    }
    else
    {
        toolBar->AddTool(id, L"", bitmap, name.c_str());
    }
}



/*
 * Name :        CFrame::SetInterface()
 * Description : Set user interface features
 * Parameters :  none
 * Returns :     none
 */
void CFrame::SetInterface()
{
    mToolBar->ToggleTool(ID_Move, mMode == ID_Move);
    mToolBar->ToggleTool(ID_Rotate, mMode == ID_Rotate);
}


/*
 * Name :        CFrame::OnMove()
 * Description : Handle a press of the Move button on the toolbar
 * Parameters :  event - The command event
 * Returns :     none
 */
void CFrame::OnMove(wxCommandEvent& event)
{
    mMode = ID_Move;
    SetInterface();
}


/*
 * Name :        CFrame::OnRotate()
 * Description : Handle a press of the rotate button on the toolbar
 * Parameters :  event - The command event
 * Returns :     none
 */
void CFrame::OnRotate(wxCommandEvent& event)
{
    mMode = ID_Rotate;
    SetInterface();
}


/*
 * Name :        CFrame::OnKeyframeSet()
 * Description : Handle a press of the keyframe set button on the toolbar
 * Parameters :  event - the command event
 * Returns :     none
 */
void CFrame::OnKeyframeSet(wxCommandEvent& event)
{
    CIteratorPtr<CActor *> iter = GetDocument()->GetActors();
    for(iter->First();  !iter->IsDone();  iter->Next())
    {
        CActor *actor = iter->Current();
        actor->SetKeyframe();
    }
}

/*
 * Name :        CFrame::OnKeyframeClear()
 * Description : Handle a press of the keyframe clear button on the toolbar
 * Parameters :  event - the command event
 * Returns :     none
 */
void CFrame::OnKeyframeClear(wxCommandEvent& event)
{
	double check = mDocument->GetTimeline()->GetCurrentTime();
    CIteratorPtr<CActor *> iter = GetDocument()->GetActors();
    for(iter->First();  !iter->IsDone();  iter->Next())
    {
        CActor *actor = iter->Current();
        actor->ClearKeyframe();
    }
    mDocument->SetAnimationTime(check);
}


/*
 * Name :        CFrame::OnExit()
 * Description : This function is called when an exit menu item is selected.
 *               It closes the main window, which ends the program.
 * Parameters :  event - An object that describes the event.
 */
void CFrame::OnExit(wxCommandEvent& event)
{
    Close(TRUE);
}


/*
 * Name :        CFrame::OnAbout()
 * Description : Menu handler for File/About menu option
 * Parameters :  event - The menu event
 * Returns :     none
 */
void CFrame::OnAbout(wxCommandEvent& event)
{
    wxMessageBox(L"The Canadian Experience is a Doc Owen Production",
                 L"About the Canadian Experience",
                  wxOK | wxICON_INFORMATION, this);
}

/*
 * Name :        CFrame::OnSave()
 * Description : Menu handler for File/Save menu option
 * Parameters :  event - The menu event
 * Returns :     none
 */
void CFrame::OnSave(wxCommandEvent& event)
{
	wxFileDialog dlg(this, L"Save File", L"", L"", L"*.canim", wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
	if(dlg.ShowModal() != wxID_OK)
		return;
	std::wstring filename = dlg.GetPath().wc_str();

	// If no suffix is supplied for the file, supply it here
	std::wstring suffix(L".canim");
	if(!equal(suffix.rbegin(), suffix.rend(), filename.rbegin()))
	{
		 filename += suffix;
	}
	SaveAnimation(filename);
}

/*
 * Name :        CFrame::SaveAnimation()
 * Description : Supports the saving of animation information to XML
 * Parameters :  none
 * Returns :     none
 */
void CFrame::SaveAnimation(const std::wstring &filename)
{
	wxXmlDocument xmlDoc;

	wxXmlNode *root = new wxXmlNode(wxXML_ELEMENT_NODE, L"animation");
	xmlDoc.SetRoot(root);

	wxXmlNode *lastChild = NULL;

	// Save frame rate, number of frames, and current time
	root->AddProperty(L"frame_rate", wxString::Format(L"%i", mDocument->GetTimeline()->GetFrameRate()));
	root->AddProperty(L"num_frames", wxString::Format(L"%i", mDocument->GetTimeline()->GetNumFrames()));
	root->AddProperty(L"curr_time", wxString::Format(L"%f", mDocument->GetTimeline()->GetCurrentTime()));

	// See how many keyframes need to be saved
	CIteratorPtr<CActor *> iter = GetDocument()->GetActors();
	int keys;
	for (iter->First(); !iter->IsDone(); iter->Next())
	{
		CActor *actor = iter->Current();
		keys = actor->XmlSaveKeyframeNum();
	}
	// Create a node for each keyframe
	for (int curr = 0; curr < keys; curr++)
	{
		wxXmlNode *keyframe = new wxXmlNode(wxXML_ELEMENT_NODE, (L"keyframe_" + wxString::Format(L"%d", curr)));
		// Save each actor's animation information
		CIteratorPtr<CActor *> iter = GetDocument()->GetActors();
		bool first = true;
		for (iter->First(); !iter->IsDone(); iter->Next())
		{
			CActor *actor = iter->Current();
			actor->XmlSave(keyframe, curr, first);
			first = false;
		}
		if (lastChild == NULL)
			root->AddChild(keyframe);
		else
			lastChild->SetNext(keyframe);
		lastChild = keyframe;
	}
    xmlDoc.Save(filename.c_str());
}

/*
 * Name :        CFrame::OnOpen()
 * Description : Menu handler for File/Open menu option
 * Parameters :  event - The menu event
 * Returns :     none
 */
void CFrame::OnOpen(wxCommandEvent& event)
{
	wxFileDialog dlg(this, L"Open File", L"", L"", L"*.canim", wxFD_OPEN | wxFD_FILE_MUST_EXIST);
	if(dlg.ShowModal() != wxID_OK)
		return;
	std::wstring filename;
	filename = dlg.GetPath().wc_str();
	OnLoad(filename);
}

/*
 * Name :        CFrame::OnLoad()
 * Description : Does actual loading of data
 * Parameters :  filename - file to load from
 * Returns :     none
 */
void CFrame::OnLoad(std::wstring filename)
{
	// Create a new Xml document for loading
	wxXmlDocument xmlDoc;

	// Unable to load such a file
	if(!xmlDoc.Load(filename.c_str()))
	{
		wstringstream str;
		str << L"Unable to open file " << filename << ends;
		wxMessageBox(str.str().c_str(), L"Error", wxOK | wxICON_ERROR);
		return;
	}

	// Dump the current channels
	CIteratorPtr<CActor *> iter = mDocument->GetActors();
	for (iter->First(); !iter->IsDone(); iter->Next())
	{
		CActor *actor = iter->Current();
		actor->ClearChannels();
	}

	// Load global animation information
	wxString frameRate = xmlDoc.GetRoot()->GetPropVal(L"frame_rate", L"");
	wxString numFrames = xmlDoc.GetRoot()->GetPropVal(L"num_frames", L"");
	wxString currTime = xmlDoc.GetRoot()->GetPropVal(L"curr_time", L"");
	CTimeline *timeline = mDocument->GetTimeline();
	timeline->SetNumFrames(wcstol(numFrames, NULL, 10));
	timeline->SetFrameRate(wcstol(frameRate, NULL, 10));

	// Now start loading actual data
	wxXmlNode *keyframe = xmlDoc.GetRoot()->GetChildren();
	bool defaultParam = true;

	// .canim structure is by keyframe
	while (keyframe != NULL)
	{
		wxString keyTime = keyframe->GetPropVal(L"time", L"");
		timeline->SetCurrentTime(wcstod(keyTime, NULL));

		// keyframe's(from .canim) children have information about/for CActor types
		wxXmlNode *actor = keyframe->GetChildren();

		while (actor != NULL)
		{
			// Convert an underscore to a space
			std::wstring name = actor->GetName().c_str();
			std::wstring spc(L" ");
			std::wstring und(L"_");
			for (unsigned int i = 0; i < name.length(); i++)
			{
				if (name[i] == und)
				{
					name[i] =spc[0];
				}
			}
			CIteratorPtr<CActor *> iter = mDocument->GetActors();
			for (iter->First(); !iter->IsDone(); iter->Next())
			{
				CActor *actors = iter->Current();
				if (name == actors->GetName())
				{
					wxString offsetX = actor->GetPropVal(L"x", L"");
					wxString offsetY = actor->GetPropVal(L"y", L"");
					wxString enabl = actor->GetPropVal(L"on", L"");
					actors->SetEnabled(wcstol(enabl, NULL, 2));
					actors->SetPosition(wxPoint(wcstol(offsetX, NULL, 10), wcstol(offsetY, NULL, 10)));
					// Use to turn on/off an actor at a keyframe
					//wxString on = actor->GetPropVal(L"enable", L"");
					//actors->SetEnabled(wcstol(on, NULL, 10));

					// Information for CActor(from .canim) contains information about/for individual drawables
					wxXmlNode *check = actor->GetChildren();

					while (check != NULL)
					{
						// Convert an underscore to a space
						std::wstring name = check->GetName().c_str();
						std::wstring spc(L" ");
						std::wstring und(L"_");
						for (unsigned int i = 0; i < name.length(); i++)
						{
							if (name[i] == und)
							{
								name[i] =spc[0];
							}
						}

						CIteratorPtr<CDrawable *> iter = actors->GetDrawables();
						for (iter->First(); !iter->IsDone(); iter->Next())
						{
							CDrawable *drawable = iter->Current();
							drawable->XmlLoad(name, defaultParam, check);
						}
						check = check->GetNext();
					}
					// Create a new keyframe for all of this new information
					actors->SetKeyframe();
				}
			}
			actor = actor->GetNext();
		}
		defaultParam = false;
		keyframe = keyframe->GetNext();
	}
	mDocument->SetAnimationTime(wcstod(currTime, NULL));
}

/*
 * Name :        CFrame::OnExpand()
 * Description : Menu handler for expanding menu option
 * Parameters :  event - The menu event
 * Returns :     none
 */
void CFrame::OnExpand(wxCommandEvent& event)
{
	// Extra logic to prevent menu from activating when a player is open
	bool tempChk = mViewFile->IsChecked(ID_Expand);
	bool temp = mViewActors->TogDisp();
	if (!temp)
	{
		mViewFile->Check(ID_Expand, !tempChk);
	}
	mViewActors->UpdateObserver();
}

/*
 * Name :        CFrame::OnAnimationProperties()
 * Description : Handle the Animation/Properties menu option.
 * Parameters :  event - The command event
 * Returns :     none
 */
void CFrame::OnAnimationProperties(wxCommandEvent &event)
{
    CTimelineDlg dlg(this, GetDocument()->GetTimeline());
    if(dlg.ShowModal() == wxOK)
    {
        GetDocument()->UpdateObservers();
    }
    GetDocument()->SetAnimationTime(0);
}

/*
 * Name :        CFrame::OnPlayer()
 * Description : Handle the Animation/Player menu option.
 * Parameters :  event - The command event
 * Returns :     none
 */
void CFrame::OnPlayer(wxCommandEvent &event)
{
	// Only allow one instance
    wxWindow *view = FindWindowById(ID_Player, this);
	mViewActors->TogActive();
	mViewEdit->TogActive();
	mViewTimeline->TogActive();
	double tempTime = mDocument->GetTimeline()->GetCurrentTime();
	mDocument->SetAnimationTime(0);
    if(view == NULL)
    {
        int wid, hit;
        GetSize(&wid, &hit);
        int ix = wid / 4;
        int iy = hit / 5;
        wxSize mSize = mDocument->GetSize();
        mPlayer = new CViewPlayer(mDocument, this, ID_Player,
        		GetPosition() + wxPoint(ix, iy), mSize,
        		mViewEdit, mViewTimeline, mViewActors, tempTime);
        mPlayer->Show(true);
        mPlayer->Raise();
    }
    else
    {
    	view->Show(true);
    	mPlayer->InitializePool();
        view->Raise();
    }
}
