/*
 * Name:     CTimelineDlg.cpp
 *
 *
 * Class:    CTimelineDlg
 * Header:   CTimelineDlg.h
 * Created : Mar 10, 2010
 * Author :  Kenji Aono
 */

#include "wx/prec.h"
#include "CDocument.h"
#include "CFrame.h"
#include "CTimelineDlg.h"
#include <sstream>
#include <vector>

using namespace std;

BEGIN_EVENT_TABLE(CTimelineDlg, wxDialog)
	EVT_BUTTON(ID_Ok, CTimelineDlg::OnOkButton)
	EVT_BUTTON(ID_Cancel, CTimelineDlg::OnCancelButton)
END_EVENT_TABLE()

/*
 * Name :        CTimelineDlg::CTimelineDlg()
 * Description : Constructor for a timeline dialog box. Initializes the parent dialog box
 *               class and constructs the controls for the dialog box.
 * Parameters :  parent - A parent window for the dialog box.
 *               timeline - The timeline class we are editing
 */
CTimelineDlg::CTimelineDlg(wxWindow *parent, CTimeline *timeline) :
    wxDialog(parent, wxID_ANY, L"Timeline Parameters", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE),
    mTimeline(timeline)
{
    // Create a vertical sizer so we have a top part of the box
    // for the options and a bottom part for the Ok and Cancel buttons
    wxBoxSizer* vSizer;
    vSizer = new wxBoxSizer( wxVERTICAL );

    // Create the grid sizer for the controls that edit the parameters
    wxGridSizer *gSizer = new wxGridSizer(2);
    vSizer->Add(gSizer, 0, wxALL | wxEXPAND, 20);
    wxStaticText *label1 = new wxStaticText( this, wxID_ANY, L"Number of Frames:", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT );
    gSizer->Add( label1, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL);

    // Convert number of frames to a string
    wstringstream str1;
    str1 << timeline->GetNumFrames();

    // Create the text control
    mNumFramesText = new wxTextCtrl( this, ID_NumFrames, str1.str().c_str(), wxDefaultPosition, wxDefaultSize, 0 );
    gSizer->Add( mNumFramesText, 0, wxALL);

    // Now add support for the frame rate
    wxStaticText *label2 = new wxStaticText( this, wxID_ANY, L"Frame Rate per Second:", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT );
    gSizer->Add( label2, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL);
    wstringstream str2;
    str2 << timeline->GetFrameRate();
    mFrameRateText = new wxTextCtrl( this, ID_FrameRate, str2.str().c_str(), wxDefaultPosition, wxDefaultSize, 0);
    gSizer->Add( mFrameRateText, 1, wxALL);

    // Create a horizontal sizer that will hold the Ok and Cancel buttons
    // and add it to the vertical sizer.
    wxBoxSizer *hSizer;
    hSizer = new wxBoxSizer(wxHORIZONTAL);
    vSizer->Add(hSizer, 0, wxALL | wxEXPAND);

    // Create an Ok button and add it to the horizontal sizer.
    mOkButton = new wxButton( this, ID_Ok, L"Ok", wxDefaultPosition, wxDefaultSize, 0 );
    hSizer->Add(mOkButton, 1, wxALL | wxEXPAND);
    mOkButton->SetDefault();

    // Create a Cancel button and add to the horizontal sizer
    mCancelButton = new wxButton( this, ID_Cancel, L"Cancel", wxDefaultPosition, wxDefaultSize, 0 );
    hSizer->Add(mCancelButton, 1, wxALL | wxEXPAND);

    // Set the top level sizer and force the window to match the size
    this->SetSizer( vSizer );
    this->Layout();
    vSizer->Fit( this );

    mNumFramesText->SetFocus();
}

/*
 * Name :        CTimelineDlg::~CTimelineDlg()
 * Description : Destructor for a timeline dialog object.
 * Parameters :  name - The name of the object we are creating.
 * Returns :     none
 */
CTimelineDlg::~CTimelineDlg()
{
}

/*
 * Name :        CTimelineDlg::OnOkButton()
 * Description : Handle an OK button press.
 * Parameters :  event - The command event
 * Returns :     none
 */
void CTimelineDlg::OnOkButton(wxCommandEvent &event)
{
    // Get the value from the control
    // mNumFramesText is of type wxTextCtrl *
    // mNumFramesText->GetValue is of type wxString
    // wxString::ToLong obtains a value as an integer.
    long numFrames;
    if(!mNumFramesText->GetValue().ToLong(&numFrames) || numFrames <= 0)
    {
        // If the value is bad, set the focus to the control and
        // return without saving the value.
        mNumFramesText->SetFocus();
        return;
    }

    long frameRate;
	if(!mFrameRateText->GetValue().ToLong(&frameRate) || frameRate <= 0)
	{
		// If the value is bad, set the focus to the control and
		// return without saving the value.
		mFrameRateText->SetFocus();
		return;
	}
	// If we are setting it to less frames, delete keyframes above it.
	if (mTimeline->GetNumFrames() > (unsigned)numFrames)
	{
		// Deal with mKeyframe 1
		mTimeline->CutChannel(numFrames);
		// And mKeyframe 2
		mTimeline->CutChannel(numFrames);
	}
    // All values have been obtained. Set them in the timeline object.
    mTimeline->SetNumFrames(numFrames);
    mTimeline->SetFrameRate(frameRate);
    mTimeline->SetCurrentTime(0);  // Also done in CFrame

    EndModal(wxOK);
}

/*
 * Name :        CTimelineDlg::OnCancelButton()
 * Description : Handle an Cancel button press.
 * Parameters :  event - The command event
 * Returns :     none
 */
void CTimelineDlg::OnCancelButton(wxCommandEvent &event)
{
    EndModal(wxCANCEL);
}
