/*
 * Name:     CAnimChannelAngle.cpp
 *
 *
 * Class:    CAnimChannelAngle
 * Header:   CAnimChannelAngle.h
 * Created : Mar 14, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CAnimChannelAngle.h"
#include "../CTimeline.h"

/*
 * Name :        CAnimChannelAngle::CAnimChannelAngle()
 * Description : Animation channel angle constructor
 */
CAnimChannelAngle::CAnimChannelAngle()
{
}

/*
 * Name :        CAnimChannelAngle::~CAnimChannelAngle()
 * Description : Animation channel angle destructor
 */
CAnimChannelAngle::~CAnimChannelAngle()
{
}

/*
 * Name :        CAnimChannelAngle::GetNumKeyframes()
 * Description : Obtain the number of keyframes. This is the size of the keyframe list.
 * Parameters :  none
 * Returns :     Number of keyframes.
 */
int CAnimChannelAngle::GetNumKeyframes()
{
    return (int)mKeyframes.size();
}

/*
 * Name :        CAnimChannelAngle::GetKeyframe()
 * Description : Get a keyframe by index from the list of keyframes.
 * Parameters :  frame - The frame index
 * Returns :     Pointer to keyframe at that index.
 */
CAnimChannel::Keyframe *CAnimChannelAngle::GetKeyframe(int frame)
{
    return &mKeyframes[frame];
}

/*
 * Name :        CAnimChannelAngle::SetKeyframe()
 * Description : Set a keyframe at the current time.
 * Parameters :  angle - The angle of the keyframe
 * Returns :     none
 */
void CAnimChannelAngle::SetKeyframe(double angle)
{
    // Create a keyframe object
    KeyframeAngle keyframe;
    keyframe.SetFrame(GetTimeline()->GetCurrentFrame());
    keyframe.SetAngle(angle);

    switch(GetInsertAction())
    {
    case Append:
        mKeyframes.push_back(keyframe);
        mKeyframe1 = (int)mKeyframes.size() - 1;
        break;

    case Replace:
        mKeyframes[mKeyframe1] = keyframe;
        break;

    case Insert:
        mKeyframes.insert(mKeyframes.begin() + mKeyframe1 + 1, keyframe);
        mKeyframe1++;
        break;
    }
}

/*
 * Name :        CAnimChannelAngle::GetAngle()
 * Description : Get the current angle.
 * Parameters :  none
 * Returns :     The angle at this point in time.
 */
double CAnimChannelAngle::GetAngle()
{
    // If no keyframes we just return zero
    if(mKeyframe1 < 0 && mKeyframe2 < 0)
        return 0;

    // If we only have one keyframe, return its value
    if(mKeyframe2 < 0)
        return mKeyframes[mKeyframe1].GetAngle();

    // Same if either is the only keyframe
    if(mKeyframe1 < 0)
        return mKeyframes[mKeyframe2].GetAngle();

    //
    // We have two keyframes. Interpolate between them.
    //

    const double PI = 3.14159265358;
    double angleFm = mKeyframes[mKeyframe1].GetAngle();
    double angleTo = mKeyframes[mKeyframe2].GetAngle();

    // Ensure we always go the short way around
    if(angleTo > angleFm)
    {
        while((angleTo - angleFm) > PI)
            angleTo -= 2 * PI;
    }
    else
    {
        while((angleTo - angleFm) < -PI)
            angleTo += 2 * PI;
    }

    // What is our t value?
    double frameRate = GetTimeline()->GetFrameRate();
    double t = (GetTimeline()->GetCurrentTime() * frameRate - mKeyframes[mKeyframe1].GetFrame()) /
                (mKeyframes[mKeyframe2].GetFrame() - mKeyframes[mKeyframe1].GetFrame());

    return angleFm + t * (angleTo - angleFm);
}

/*
 * Name :        CAnimChannelAngle::ClearKeyframe()
 * Description : Determines if a keyframe should be removed
 * Parameters :  none
 * Returns :     none
 */
void CAnimChannelAngle::ClearKeyframe()
{
	int currFrame = GetTimeline()->GetCurrentFrame();
	if (mKeyframe2 != -1 && currFrame == mKeyframes[mKeyframe2].GetFrame())
	{
		mKeyframes.erase(mKeyframes.begin() + mKeyframe2);
		if (mKeyframe2 < int(mKeyframes.size()))
			mKeyframe2++;
		else
			mKeyframe2 = -1;
	}
	if (mKeyframe1 != -1 && currFrame == mKeyframes[mKeyframe1].GetFrame())
	{
		mKeyframes.erase(mKeyframes.begin() + mKeyframe1);
		if (mKeyframes.size() > 0)
			mKeyframe1--;
		else
			mKeyframe1 = -1;
	}
}

/*
 * Name :        CAnimChannelAngle::EmptyChannel()
 * Description : Clears all keyframes
 * Parameters :  none
 * Returns :     none
 */
void CAnimChannelAngle::EmptyChannel()
{
	int index = mKeyframes.size();
	while (index > 0)
	{
		mKeyframes.erase(mKeyframes.begin() + (index - 1));
		index--;
	}
    mKeyframe1 = -1;
    mKeyframe2 = -1;
}
