/*
 * Name:     CAnimChannelPos.cpp
 *
 *
 * Class:    CAnimChannelPos
 * Header:   CAnimChannelPos.h
 * Created : Mar 14, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CAnimChannelPos.h"
#include "../CTimeline.h"

/*
 * Name :        CAnimChannelPos::CAnimChannelPos()
 * Description : Animation channel position constructor
 */
CAnimChannelPos::CAnimChannelPos()
{
}

/*
 * Name :        CAnimChannelPos::~CAnimChannelPos()
 * Description : Animation channel position destructor
 */
CAnimChannelPos::~CAnimChannelPos()
{
}

/*
 * Name :        CAnimChannelPos::GetNumKeyframes()
 * Description : Obtain the number of keyframes. This is the size of the keyframe list.
 * Parameters :  none
 * Returns :     Number of keyframes.
 */
int CAnimChannelPos::GetNumKeyframes()
{
    return (int)mKeyframes.size();
}

/*
 * Name :        CAnimChannelPos::GetKeyframe()
 * Description : Get a keyframe by index from the list of keyframes.
 * Parameters :  frame - The frame index
 * Returns :     Pointer to keyframe at that index.
 */
CAnimChannel::Keyframe *CAnimChannelPos::GetKeyframe(int frame)
{
    return &mKeyframes[frame];
}

/*
 * Name :        CAnimChannelPos::SetKeyframe()
 * Description : Set a keyframe at the current time.
 * Parameters :  pos - The position of the keyframe
 * Returns :     none
 */
void CAnimChannelPos::SetKeyframe(wxPoint pos)
{
    // Create a keyframe object
    KeyframePos keyframe;
    keyframe.SetFrame(GetTimeline()->GetCurrentFrame());
    keyframe.SetPos(pos);

    switch(GetInsertAction())
    {
    case Append:
        mKeyframes.push_back(keyframe);
        mKeyframe1 = (int)mKeyframes.size() - 1;
        break;

    case Replace:
        mKeyframes[mKeyframe1] = keyframe;
        break;

    case Insert:
        mKeyframes.insert(mKeyframes.begin() + mKeyframe1 + 1, keyframe);
        mKeyframe1++;
        break;
    }
}

/*
 * Name :        CAnimChannelPos::GetPos()
 * Description : Get the current position.
 * Parameters :  none
 * Returns :     The position at this point in time.
 */
wxPoint CAnimChannelPos::GetPos(wxPoint old)
{
    // If no keyframes we don't move anything
    if(mKeyframe1 < 0 && mKeyframe2 < 0)
        return old;

    // If we only have one keyframe, return its value
    if(mKeyframe2 < 0)
        return mKeyframes[mKeyframe1].GetPos();

    // Same if either is the only keyframe
    if(mKeyframe1 < 0)
        return mKeyframes[mKeyframe2].GetPos();

    //
    // We have two keyframes. Interpolate between them.
    //
    wxPoint posFm = mKeyframes[mKeyframe1].GetPos();
    wxPoint posTo = mKeyframes[mKeyframe2].GetPos();

    // Find a t value
    double frameRate = GetTimeline()->GetFrameRate();
    double t = (GetTimeline()->GetCurrentTime() * frameRate - mKeyframes[mKeyframe1].GetFrame()) /
                (mKeyframes[mKeyframe2].GetFrame() - mKeyframes[mKeyframe1].GetFrame());

    wxPoint posNow(int(posFm.x + t * (posTo.x - posFm.x)),
    		       int(posFm.y + t * (posTo.y - posFm.y)));

    return posNow;
}

/*
 * Name :        CAnimChannelPos::ClearKeyframe()
 * Description : Determines if a keyframe should be removed
 * Parameters :  none
 * Returns :     none
 */
void CAnimChannelPos::ClearKeyframe()
{
	int currFrame = GetTimeline()->GetCurrentFrame();
	if (mKeyframe2 != -1 && currFrame == mKeyframes[mKeyframe2].GetFrame())
	{
		mKeyframes.erase(mKeyframes.begin() + mKeyframe2);
		if (mKeyframe2 < int(mKeyframes.size()))
			mKeyframe2++;
		else
			mKeyframe2 = -1;
	}
	if (mKeyframe1 != -1 && currFrame == mKeyframes[mKeyframe1].GetFrame())
	{
		mKeyframes.erase(mKeyframes.begin() + mKeyframe1);
		if (mKeyframes.size() > 0)
			mKeyframe1--;
		else
			mKeyframe1 = -1;
	}
}

/*
 * Name :        CAnimChannelPos::EmptyChannel()
 * Description : Clears all keyframes
 * Parameters :  none
 * Returns :     none
 */
void CAnimChannelPos::EmptyChannel()
{
	int index = mKeyframes.size();
	while (index > 0)
	{
		mKeyframes.erase(mKeyframes.begin() + (index - 1));
		index--;
	}
    mKeyframe1 = -1;
    mKeyframe2 = -1;
}
