/*
 * Name:     CDrawable.cpp
 *
 *
 * Class:    CDrawable
 * Header:   CDrawable.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "../CActor.h"
#include "CDrawable.h"
#include <cmath>

/*
 * Name :        CDrawable::CDrawable()
 * Description : Drawable class constructor
 */
CDrawable::CDrawable()
{
}

/*
 * Name :        CDrawable::CDrawable()
 * Description : Constructor for a drawable.
 * Parameters :  name - the name of the drawable.
 * Returns :     none
 */
CDrawable::CDrawable(const std::wstring &name) : mName(name)
{
    mPosition = wxPoint(0, 0);
    mRotation = 0;
    mPlacedPosition = wxPoint(0, 0);
    mPlacedR = 0;
    mActor = NULL;
    mParent = NULL;
    mEnabled = true;
}

/*
 * Name :        CDrawable::!CDrawable()
 * Description : Drawable class destructor
 */
CDrawable::~CDrawable()
{
	for(std::list<CDrawable *>::iterator c=mChildren.begin(); c!=mChildren.end(); c++)
	{
		delete *c;
	}

	mChildren.clear();
}

/*
 * Name :        CDrawable::Place()
 * Description : Place the drawables at the correct location. This works heirarchically
 *               from top item down.
 * Parameters :  offset - An x,y offset
 *               rotation - A rotation
 * Returns :     none
 */
void CDrawable::Place(wxPoint offset, double rotate)
{
    // Combine the transformation we are given with the transformation
    // for this object.
    mPlacedPosition = offset + RotatePoint(mPosition, rotate);
    mPlacedR = mRotation + rotate;

    // Update our children
    for(std::list<CDrawable *>::iterator d=mChildren.begin(); d!=mChildren.end(); d++)
    {
        (*d)->Place(mPlacedPosition, mPlacedR);
    }
}

/*
 * Name :        CDrawable::AddChild()
 * Description : Add a child to his drawable's composite pattern.
 * Parameters :  child - The child to add.
 * Returns :     none
 */
void CDrawable::AddChild(CDrawable *child)
{
	mChildren.push_back(child);
	child->SetParent(this);
}

/*
 * Name :        CDrawable::AddSChild()
 * Description : Add a child to his drawable's composite pattern without setting default (for CHead)
 * Parameters :  child - The child to add.
 * Returns :     none
 */
void CDrawable::AddSChild(CDrawable *child)
{
	mChildren.push_back(child);
	child->SetParent(this);
}

/*
 * Name :        CDrawable::IsMovable()
 * Description : Determine if a drawable is moveable. By default most are not movable.
 * Parameters :  none
 * Returns :     true if object is movable.
 */
bool CDrawable::IsMovable()
{
    return false;
}

/*
 * Name :        CDrawable::Move()
 * Description : Move the object relative to it's parent by a delta in screen coordinates.
 * Parameters :  delta - Distance to move.
 * Returns :     none
 */
void CDrawable::Move(wxPoint delta)
{
	// Need to account for a parent or root having a rotation, and adjust delta
	// Follow how changing an angle works, a mouse up always moves the
	// drawable 'up' if a parent's rotation is considered as zero
	CDrawable* tempChild = GetParent();
	// The total angle away from zero(up), +(clockwise) -(counter-clockwise)
	double tempRot = 0;
	tempRot += GetActor()->GetRoot()->GetRotation();
	if (mName == std::wstring(L"Text Bubble")) {tempRot = 0;}
	const double PI = 3.14159265358;
	while (tempChild != NULL && tempChild->GetParent() != NULL)
	{
		tempRot += tempChild->GetRotation();
		tempChild = tempChild->GetParent();
		tempRot = fmod(tempRot, 2*PI);
	}
	// Calculate offset considering rotations of objects
	wxPoint newDel( ((delta.x * cos(tempRot)) + (delta.y * cos(tempRot + PI/2))),
	                ((delta.y * cos(tempRot)) + (delta.x * cos(tempRot - PI/2))));
	SetPosition(wxPoint((GetPosition().x+ newDel.x), (GetPosition().y + newDel.y)));
}

/*
 * Name :        CDrawable::RotatePoint()
 * Description : Rotate a point by a given angle.
 * Parameters :  point - The point to rotate
 *               angle - An angle in radians
 * Returns :     Rotated point
 *
 * Note:         This functions such that moving the cursor
 *               up rotates clockwise, and down counter-clockwise
 */
wxPoint CDrawable::RotatePoint(wxPoint point, double angle)
{
    double cosA = cos(angle);
    double sinA = sin(angle);

    return wxPoint(cosA * point.x + sinA * point.y,
                  -sinA * point.x + cosA * point.y);
}

/*
 * Name :        CDrawable::SetActor()
 * Description : Set the actor that uses this drawable.
 * Parameters :  actor - The actor that uses us.
 * Returns :     none
 */
void CDrawable::SetActor(CActor *actor)
{
    mActor = actor;
    mDefaultPos = mPosition;
    mDefaultRot = mRotation;
    // Set the channel name
    mChannelA.SetName(actor->GetName() + L":" + mName);
    mChannelP.SetName(actor->GetName() + L":" + mName);
    mChannelE.SetName(actor->GetName() + L":" + mName);
}

/*
 * Name :        CDrawable::SetTimeline()
 * Description : Set the timeline for all channels.
 * Parameters :  timeline - The timeline class.
 * Returns :     none
 */
void CDrawable::SetTimeline(CTimeline *timeline)
{
    timeline->AddChannel(&mChannelA);
    timeline->AddChannel(&mChannelP);
    timeline->AddChannel(&mChannelE);
}

/*
 * Name :        CDrawable::SetKeyframe()
 * Description : Set the keyframe based on the current status.
 * Parameters :  none
 * Returns :     none
 */
void CDrawable::SetKeyframe()
{
    mChannelA.SetKeyframe(mRotation);
    mChannelP.SetKeyframe(mPosition);
    mChannelE.SetKeyframe(mEnabled);
}

/*
 * Name :        CDrawable::GetKeyframe()
 * Description : Get the current channel from the animation system.
 * Parameters :  none
 * Returns :     none
 */
void CDrawable::GetKeyframe()
{
	if (mChannelA.GetNumKeyframes() > 0)
	{
		mRotation = mChannelA.GetAngle();
	}
	if (mChannelP.GetNumKeyframes() > 0)
	{
		mPosition = mChannelP.GetPos(mPosition);
	}
	if (mChannelE.GetNumKeyframes() > 0)
	{
		mEnabled = mChannelE.GetEnab(mEnabled);
	}
}

/*
 * Name :        CDrawable::ClearKeyframe()
 * Description : Clear the current frame from the animation system.
 * Parameters :  none
 * Returns :     none
 */
void CDrawable::ClearKeyframe()
{
    mChannelA.ClearKeyframe();
    mChannelP.ClearKeyframe();
    mChannelE.ClearKeyframe();
}

/*
 * Name :        CDrawable::ClearChannels()
 * Description : Clear the channel of all keyframes
 * Parameters :  none
 * Returns :     none
 */
void CDrawable::ClearChannels()
{
	mChannelA.EmptyChannel();
	mChannelP.EmptyChannel();
	mChannelE.EmptyChannel();
}

/*
 * Name :        CDrawable::XmlSaveDrawable()
 * Description : Saves information for a drawable
 * Parameters :  The actor it belongs to
 * Returns :     none
 */
void CDrawable::XmlSaveDrawable(wxXmlNode* actor)
{
	// Convert any spaces to an underscore
	std::wstring name = GetName();
	std::wstring spc(L" ");
	std::wstring und(L"_");
	for (unsigned int i = 0; i < name.length(); i++)
	{
		if (name[i] == spc)
		{
			name[i] =und[0];
		}
	}

	// Create a node for this drawable item
	wxXmlNode *drawable = new wxXmlNode(wxXML_ELEMENT_NODE, name);

	// Add a position if applicable to save space
	if (IsMovable())
	{
		//wxXmlNode *position = new wxXmlNode(wxXML_ELEMENT_NODE, L"position");
		wxPoint tempPos = mChannelP.GetPos(mPosition);
		drawable->AddProperty(L"x", wxString::Format(L"%d", tempPos.x));
		drawable->AddProperty(L"y", wxString::Format(L"%d", tempPos.y));
		//drawable->AddChild(position);
	}

	// Add angle information (default to zero?)
	//wxXmlNode *angle = new wxXmlNode(wxXML_ELEMENT_NODE, L"angle");
	double tempAng = mChannelA.GetAngle();
	// Only save two decimal points for angles since the rest are 0
	drawable->AddProperty(L"angle", wxString::Format(L"%.2f", tempAng));
	//drawable->AddChild(angle);
	bool tempEnab = mChannelE.GetEnab(mEnabled);
	drawable->AddProperty(L"on", wxString::Format(L"%d", tempEnab));

	actor->AddChild(drawable);
}


/*
 * Name :        CDrawable::XmlLoad()
 * Description : Loads information about the drawable from file
 * Parameters :  the name of the drawable to be loading
 * Returns :     none
 */
void CDrawable::XmlLoad(std::wstring name, bool defaultParam, wxXmlNode *check)
{
	if (name == mName)
	{
		wxString rot = check->GetPropVal(L"angle", L"");
		wxString enb = check->GetPropVal(L"on", L"");
		SetEnabled(wcstol(enb, NULL, 2));
		if (IsRotatable())
		{
			SetRotation(wcstod(rot, NULL));
		}
		if (IsMovable())
		{
			wxString X = check->GetPropVal(L"x", L"");
			wxString Y = check->GetPropVal(L"y", L"");
			SetPosition(wxPoint(wcstol(X, NULL, 10), wcstol(Y, NULL, 10)));
		}
		if (defaultParam)
		{
			mDefaultPos = GetPosition();
			mDefaultRot = GetRotation();
		}
	}
}

/*
 * Name :        CDrawable::SetPosition()
 * Description : Sets the position
 * Parameters :  The new position
 * Returns :     none
 */
void CDrawable::SetPosition(wxPoint p)
{
	mPosition = p;
}

/*
 * Name :        CDrawable::SetRotation()
 * Description : Sets the rotation
 * Parameters :  The new angle
 * Returns :     none
 */
void CDrawable::SetRotation(double r)
{
	mRotation = r;
}

