/*
 * Name:     CTextBubbleDrawable.cpp
 *
 *
 * Class:    CTextBubbleDrawable
 * Header:   CTextBubbleDrawable.h
 * Created : Mar 21, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "../CFrame.h"
#include "CTextBubbleDlg.h"
#include "CTextBubbleDrawable.h"
#include <sstream>

/*
 * Name :        CTextBubbleDrawable::CTextBubbleDrawable()
 * Description : Constructor for a text bubble drawable object.
 * Parameters :  name - The name of the object we are creating.
 * Returns :     none
 */
CTextBubbleDrawable::CTextBubbleDrawable(const std::wstring &name) : CDrawable(name)
{
	SetMirror(true);
}

/*
 * Name :        CTextBubbleDrawable::~CTextBubbleDrawable()
 * Description : CTextBubbleDrawable class destructor.
 */
CTextBubbleDrawable::~CTextBubbleDrawable()
{
}

/*
 * Name :        CTextBubbleDrawable::Draw()
 * Description : Draw the drawable image.
 * Parameters :  dc - Device context
 * Returns :     none
 */
void CTextBubbleDrawable::Draw(wxDC* dc)
{
	if (!mEnabled )
		return;
	// Move the text bubble with the actor
	SetPosition(wxPoint((GetX() + mPlacedPosition.x ), (GetY() + mPlacedPosition.y )));
	DrawMe(dc);
	SetPosition(wxPoint((GetX() - mPlacedPosition.x ), (GetY() - mPlacedPosition.y )));
}

/*
 * Name :        CTextBubbleDrawable::SetActor()
 * Description : Set the actor that uses this drawable.
 * Parameters :  actor - The actor that uses us.
 * Returns :     none
 */
void CTextBubbleDrawable::SetActor(CActor *actor)
{
    mActor = actor;
    mDefaultPos = GetPosition();
    mDefaultText = GetText();
    mMirr = GetMirror();
    // Set the channel name
    mChannelTB.SetName(actor->GetName() + L":" + mName);
    mChannelP.SetName(actor->GetName() + L":" + mName);
    mChannelM.SetName(actor->GetName() + L":" + mName);
}

/*
 * Name :        CTextBubbleDrawable::HitTest()
 * Description : Test to see if we hit this object with a mouse.
 * Parameters :  pos - x,y postion
 * Returns :     true if hit.
 */
bool CTextBubbleDrawable::HitTest(wxPoint pos)
{
	return IsPointInside((pos.x - mPlacedPosition.x), (pos.y - mPlacedPosition.y));
}

/*
 * Name :        CTextBubbleDrawable::SetPosition()
 * Description : Updates the position of the drawable text bubble
 * Parameters :  pos - x,y postion
 * Returns :     none
 */
void CTextBubbleDrawable::SetPosition(wxPoint p)
{
	SetBubblePosition(p.x, p.y);
}

/*
 * Name :        CTextBubbleDrawable::DispDiag()
 * Description : Calls CTextBubbleDlg
 * Parameters :  The source frame
 * Returns :     none
 */
void CTextBubbleDrawable::DispDiag(CFrame *frame)
{
	//double tempTime = GetActor()->GetDocument()->GetTimeline()->GetCurrentTime();
    CTextBubbleDlg dlg(frame, this);
    if(dlg.ShowModal() == wxOK)
    {
    	GetActor()->GetDocument()->UpdateObservers();
    }
    //GetActor()->GetDocument()->SetAnimationTime(tempTime);
}

/*
 * Name :        CTextBubbleDrawable::SetTimeline()
 * Description : Set the timeline for all channels.
 * Parameters :  timeline - The timeline class.
 * Returns :     none
 */
void CTextBubbleDrawable::SetTimeline(CTimeline *timeline)
{
    timeline->AddChannel(&mChannelTB);
    timeline->AddChannel(&mChannelP);
    timeline->AddChannel(&mChannelM);
    timeline->AddChannel(&mChannelE);
}

/*
 * Name :        CTextBubbleDrawable::SetKeyframe()
 * Description : Set the keyframe based on the current status.
 * Parameters :  none
 * Returns :     none
 */
void CTextBubbleDrawable::SetKeyframe()
{
    mChannelTB.SetKeyframe(GetText());
    mChannelP.SetKeyframe(GetPosition());
    mChannelM.SetKeyframe(GetMirr());
    mChannelE.SetKeyframe(mEnabled);
}

/*
 * Name :        CTextBubbleDrawable::GetKeyframe()
 * Description : Get the current channel from the animation system.
 * Parameters :  none
 * Returns :     none
 */
void CTextBubbleDrawable::GetKeyframe()
{
	if (mChannelTB.GetNumKeyframes() > 0)
	{
		SetText(mChannelTB.GetText(GetText()));
	}
	if (mChannelP.GetNumKeyframes() > 0)
	{
		SetPosition(mChannelP.GetPos(GetPosition()));
	}
	if (mChannelM.GetNumKeyframes() > 0)
	{
		SetMirr(mChannelM.GetText(GetMirr()));
	}
	if (mChannelE.GetNumKeyframes() > 0)
	{
		mEnabled = mChannelE.GetEnab(mEnabled);
	}
}

/*
 * Name :        CTextBubbleDrawable::ClearKeyframe()
 * Description : Clear the current frame from the animation system.
 * Parameters :  none
 * Returns :     none
 */
void CTextBubbleDrawable::ClearKeyframe()
{
    mChannelTB.ClearKeyframe();
    mChannelP.ClearKeyframe();
    mChannelM.ClearKeyframe();
    mChannelE.ClearKeyframe();
}

/*
 * Name :        CTextBubbleDrawable::ClearChannels()
 * Description : Clear the channel of all keyframes
 * Parameters :  none
 * Returns :     none
 */
void CTextBubbleDrawable::ClearChannels()
{
	mChannelTB.EmptyChannel();
	mChannelP.EmptyChannel();
	mChannelM.EmptyChannel();
	mChannelE.EmptyChannel();
}

/*
 * Name :        CTextBubbleDrawable::GetMirr()
 * Description : Finds the state of the mirror
 * Parameters :  none
 * Returns :     String indicating result
 */
std::wstring CTextBubbleDrawable::GetMirr()
{
	if (GetMirror() == true)
		return std::wstring(L"M");
	else
		return std::wstring(L"N");
}

/*
 * Name :        CTextBubbleDrawable::SetMirr()
 * Description : Set mMirr based on string
 * Parameters :  none
 * Returns :     none
 */
void CTextBubbleDrawable::SetMirr(std::wstring check)
{
	if (check == std::wstring(L"M"))
		SetMirror(true);
	else
		SetMirror(false);
}

/*
 * Name :        CTextBubbleDrawable::XmlSaveDrawable()
 * Description : Saves information for a drawable
 * Parameters :  The actor it belongs to
 * Returns :     none
 */
void CTextBubbleDrawable::XmlSaveDrawable(wxXmlNode* actor)
{
	// Convert any spaces to an underscore
	std::wstring name = GetName();
	std::wstring spc(L" ");
	std::wstring und(L"_");
	for (unsigned int i = 0; i < name.length(); i++)
	{
		if (name[i] == spc)
		{
			name[i] =und[0];
		}
	}

	// Create a node for this drawable item
	wxXmlNode *drawable = new wxXmlNode(wxXML_ELEMENT_NODE, name);

	wxPoint tempPos = mChannelP.GetPos(GetPosition());
	drawable->AddProperty(L"x", wxString::Format(L"%d", tempPos.x));
	drawable->AddProperty(L"y", wxString::Format(L"%d", tempPos.y));

	bool tempEnab = mChannelE.GetEnab(mEnabled);
	drawable->AddProperty(L"on", wxString::Format(L"%d", tempEnab));

	drawable->AddProperty(L"mirror", mChannelM.GetText(GetMirr()));

	drawable->AddProperty(L"text", mChannelTB.GetText(mDefaultText));

	actor->AddChild(drawable);
}

/*
 * Name :        CTextBubbleDrawable::XmlLoad()
 * Description : Loads information about the drawable from file
 * Parameters :  the name of the drawable to be loading
 * Returns :     none
 */
void CTextBubbleDrawable::XmlLoad(std::wstring name, bool defaultParam, wxXmlNode *check)
{
	if (name == mName)
	{
		wxString X = check->GetPropVal(L"x", L"");
		wxString Y = check->GetPropVal(L"y", L"");
		SetPosition(wxPoint(wcstol(X, NULL, 10), wcstol(Y, NULL, 10)));
		wxString enb = check->GetPropVal(L"on", L"");
		SetEnabled(wcstol(enb, NULL, 2));
		wxString mirr = check->GetPropVal(L"mirror", L"");
		wxString text = check->GetPropVal(L"text", L"");
		SetText(text.c_str());
		SetMirr(mirr.c_str());
		if (defaultParam)
		{
			mDefaultPos = GetPosition();
			mDefaultText = GetText();
			mMirr = GetMirror();
		}
	}
}
