/*
 * Name:     CViewEdit.cpp
 * The main window for editing our presentation.
 *
 * Class:    CViewEdit
 * Header:   CViewEdit.h
 * Created : Feb 20, 2010
 * Author :  Charles B. Owen - Initial distributed version
 * Co-Author: Kenji Aono
 */

#include "../wx/prec.h"
#include "CViewEdit.h"
#include "../CFrame.h"
#include "../CDocument.h"

const double RotationScaling = 0.02;

BEGIN_EVENT_TABLE(CViewEdit, wxScrolledWindow)
    EVT_PAINT(CViewEdit::OnPaint)
    EVT_LEFT_DOWN(CViewEdit::OnLeftButtonDown)
    EVT_LEFT_UP(CViewEdit::OnMouseMove)
    EVT_RIGHT_DOWN(CViewEdit::OnRightButtonDown)
    EVT_MOTION(CViewEdit::OnMouseMove)
END_EVENT_TABLE()

/*
 * Name :        CViewEdit::CViewEdit()
 * Description : Constructor for an editor view.
 * Parameters :  frame - The frame this view is a member of. The frame owns the document.
 * Returns :     none
 */
CViewEdit::CViewEdit(CFrame *frame) :
    wxScrolledWindow(frame, wxID_ANY, wxDefaultPosition, wxSize(805, 605), wxHSCROLL | wxVSCROLL | wxBORDER_SUNKEN),
    CDocumentObserver(frame->GetDocument()),
    mFrame(frame)
{
    SetBackgroundColour(wxColour(128, 128, 128));
    SetVirtualSize(GetDocument()->GetSize());
    SetClientSize(GetDocument()->GetSize());
    SetScrollRate(1, 1);
    mLastMouse = wxPoint(0, 0);
    mSelectedActor = NULL;
    mSelectedDrawable = NULL;
    mActive = true;
}

/*
 * Name :        CViewEdit::~CViewEdit()
 * Description : Destructor.
 */
CViewEdit::~CViewEdit()
{

}

/*
 * Name :        CViewEdit::OnPaint()
 * Description : Handle a paint request for this window.
 * Parameters :  event - The paint event
 * Returns :     none
 */
void CViewEdit::OnPaint(wxPaintEvent &event)
{
    // Create a device context
    wxPaintDC dc(this);
    dc.SetClippingRegion(wxPoint(0, 0), GetDocument()->GetSize());

    // Offset the device context due to any scrolling
    DoPrepareDC(dc);

    GetDocument()->Draw(&dc);

    dc.SetPen(wxNullPen);
    dc.SetBrush(wxNullBrush);
    dc.SetFont(wxNullFont);
}

/*
 * Name :         CViewEdit::OnLeftButtonDown()
 * Description :  This function is called when the left mouse button is clicked
 *                on the window.
 */
void CViewEdit::OnLeftButtonDown(wxMouseEvent &event)
{
	if (!mActive)
		return;
    // Get the view point start and add to the mouse position
    // to determine where we clicked in the scrolled area.
    int x, y;
    GetViewStart(&x, &y);
    x += event.m_x;
    y += event.m_y;

    mLastMouse = wxPoint(x, y);

    //
    // Did we hit anything?
    //

    CIteratorPtr<CActor *> iter = GetDocument()->GetActors();
    CActor *hitActor = NULL;
    CDrawable *hitDrawable = NULL;
    for(iter->First();  !iter->IsDone();  iter->Next())
    {
        // Note: We do not exit when we get the first hit, since
        // we are looking at these in drawing order. Instead, we
        // keep the last hit.
        CActor *actor = iter->Current();
        CDrawable *drawable = actor->HitTest(wxPoint(x, y));
        if(drawable != NULL)
        {
            hitActor = actor;
            hitDrawable = drawable;
        }
    }

    // If we hit something determine what we do with it based on the
    // current mode.
    if(hitActor != NULL)
    {
        mSelectedActor = hitActor;
        mSelectedDrawable = hitDrawable;
    }
}

/*
 * Name :        CViewEdit::OnMouseMove()
 * Description : This function is called whenever the mouse is
 *               moved over the window.
 */
void CViewEdit::OnMouseMove(wxMouseEvent &event)
{
	if (!mActive)
		return;
    // Convert the mouse location to a canvas location
    int x, y;
    GetViewStart(&x, &y);
    x += event.m_x;
    y += event.m_y;

    // This code computes a "delta", the amount the mouse has moved
    // since the last call to button down or mouse move.
    wxPoint mouse = wxPoint(x, y);
    wxPoint delta = mouse - mLastMouse;
    mLastMouse = mouse;

    if(event.m_leftDown)
    {
        switch(mFrame->GetMode())
        {
        case CFrame::ID_Move:
            if(mSelectedDrawable != NULL)
            {
                if(mSelectedDrawable->IsMovable())
                {
                    mSelectedDrawable->Move(delta);
                }
                else
                {
                    mSelectedActor->SetPosition(mSelectedActor->GetPosition() + delta);
                }
                mFrame->GetDocument()->UpdateObservers();
            }
            break;

        case CFrame::ID_Rotate:
            if(mSelectedDrawable != NULL && mSelectedDrawable->IsRotatable())
            {
                mSelectedDrawable->SetRotation(mSelectedDrawable->GetRotation() + delta.y * RotationScaling);
                mFrame->GetDocument()->UpdateObservers();
            }
            break;

        default:
            break;
        }

    }
    else
    {
        mSelectedDrawable = NULL;
        mSelectedActor = NULL;
    }
}

/*
 * Name :        CViewEdit::UpdateObserver()
 * Description : Force an update of this window when the document changes.
 * Parameters :  none
 * Returns :     none
 */
void CViewEdit::UpdateObserver()
{
	if (mActive)
		Refresh();
}

/*
 * Name :         CViewEdit::OnRightButtonDown()
 * Description :  This function is called when the right mouse button is clicked
 *                on the window.
 */
void CViewEdit::OnRightButtonDown(wxMouseEvent &event)
{
	if (!mActive)
		return;

	// Calls to display dialogs for Text Bubbles and Facial Expressions
	// Get the view point start and add to the mouse position
	// to determine where we clicked in the scrolled area.
	int x, y;
	GetViewStart(&x, &y);
	x += event.m_x;
	y += event.m_y;

	mLastMouse = wxPoint(x, y);

	//
	// Did we hit anything?
	//

	CIteratorPtr<CActor *> iter = GetDocument()->GetActors();
	CActor *hitActor = NULL;
	CDrawable *hitDrawable = NULL;
	for(iter->First();  !iter->IsDone();  iter->Next())
	{
		// Note: We do not exit when we get the first hit, since
		// we are looking at these in drawing order. Instead, we
		// keep the last hit.
		CActor *actor = iter->Current();
		CDrawable *drawable = actor->HitTest(wxPoint(x, y));
		if(drawable != NULL)
		{
			hitActor = actor;
			hitDrawable = drawable;
		}
	}

	// If we hit something try to display it's dialog box
	if(hitActor != NULL)
	{
		hitDrawable->DispDiag(mFrame);
	}
}
