/*
 * Name:     CViewPlayer.cpp
 *
 *
 * Class:    CViewPlayer
 * Header:   CViewPlayer.h
 * Created : Mar 21, 2010
 * Author :  Kenji Aono
 */

#include "../wx/prec.h"
#include "CViewPlayer.h"

using namespace std;

BEGIN_EVENT_TABLE(CViewPlayer, wxFrame)
    EVT_PAINT(CViewPlayer::OnPaint)
    EVT_CLOSE(CViewPlayer::OnExit)
    EVT_TIMER(ID_Timer, CViewPlayer::OnTimer)
    EVT_BUTTON(ID_Play, CViewPlayer::OnPlay)
    EVT_BUTTON(ID_Pause, CViewPlayer::OnPause)
    EVT_BUTTON(ID_Stop, CViewPlayer::OnStop)
    EVT_BUTTON(ID_Close, CViewPlayer::OnClose)
END_EVENT_TABLE()

/*
 * Name :        CViewPlayer::CViewPlayer()
 * Description : Constructor for a player view.
 * Parameters :  frame - The frame this view is a member of. The frame owns the document.
 * Returns :     none
 */
CViewPlayer::CViewPlayer(CDocument *document, wxWindow *parent, wxWindowID id,
		const wxPoint &pos, const wxSize &size, CViewEdit *vEdit,
		CViewTimeline *vTime, CViewActors *vAct, double tempTime) :
		wxFrame(parent, id, L"Animation Player", pos, size, wxCAPTION | wxCLOSE_BOX),
		CDocumentObserver(document), mDocument(document), mParent(parent), mTimer(this, ID_Timer)
{
	wxSize tempSize = mDocument->GetSize();
	tempSize.SetHeight(tempSize.GetHeight() + 25);
	SetClientSize(tempSize);
	// Gray background to see if size is correct
	SetBackgroundColour(wxColor(128, 128, 128));
	mViewEdit = vEdit;
	mViewTimeline = vTime;
	mViewActors = vAct;
	mTempTime = tempTime;
	mPlaying = false;
	InitializePool();

	// The following is to add the buttons below the animated view
    wxBoxSizer *vertSizer = new wxBoxSizer(wxVERTICAL);
    vertSizer->AddSpacer(tempSize.GetHeight());        // horzSizer is first (top) in vertSizer
    SetSizer(vertSizer);

    mPlay = new wxButton(this, ID_Play, wxT("Play"),  wxPoint(0, tempSize.GetHeight() - 25),
    		              wxSize((tempSize.GetWidth() / 2), 25));
    mPause = new wxButton(this, ID_Pause, wxT("Pause"),  wxPoint(0, tempSize.GetHeight() - 25),
    		              wxSize((tempSize.GetWidth() / 2), 25));
    mStop = new wxButton(this, ID_Stop, wxT("Stop"),  wxPoint(2*(tempSize.GetWidth() / 4),
    		               tempSize.GetHeight() - 25), wxSize((tempSize.GetWidth() / 4), 25));
    mClose = new wxButton(this, ID_Close, wxT("Close"),  wxPoint(3*(tempSize.GetWidth() / 4),
    		               tempSize.GetHeight() - 25), wxSize((tempSize.GetWidth() / 4), 25));

    mPause->Show(false);
}

/*
 * Name :        CViewPlayer::CViewPlayer()
 * Description : Destructor for a player view.
 * Parameters :  frame - The frame this view is a member of. The frame owns the document.
 * Returns :     none
 */
CViewPlayer::~CViewPlayer()
{
}

/*
 * Name :        CViewPlayer::UpdateObserver
 * Description : Will invalidate screen, and force an update
 * Parameters :  none
 * Returns :     void
 */
void CViewPlayer::UpdateObserver()
{
	Refresh();
}

/*
 * Name :        CViewPlayer::OnPaint
 * Description : This will create a DC, and draw required components
 * Parameters :  The object for a paint event
 * Returns :     void
 */
void CViewPlayer::OnPaint(wxPaintEvent &event)
{
    // Create a device context
    wxPaintDC dc(this);
    dc.SetClippingRegion(wxPoint(0, 0), GetDocument()->GetSize());

    GetDocument()->Draw(&dc);

    mUsed.DrawParticles(&dc);

    dc.SetPen(wxNullPen);
    dc.SetBrush(wxNullBrush);
    dc.SetFont(wxNullFont);
}

/*
 * Name :        CViewPlayer::OnTimer
 * Description : This will adjust the time for the animation
 * Parameters :  The object for a command event
 * Returns :     void
 */
void CViewPlayer::OnTimer(wxTimerEvent& event)
{
	mUsed.UpdatePosition(&mUnUsed);  // This also does CleanSnow's job
	CreateMoreSnow();
	// Set to the elapsed time, since updates may take longer than timer interval
	mDocument->SetAnimationTime(mTrueTime.Time() / 1000.0);
	if ((mTrueTime.Time() / 1000.0) > (mDocument->GetTimeline()->GetDuration() + 0.01) )
	{
		mTimer.Stop();
		mTrueTime.Pause();
		mPause->Show(false);
		mPlay->Show(true);
		mPlaying = false;
	}
}

/*
 * Name :        CViewPlayer::OnPlay
 * Description : This will play the animation
 * Parameters :  The object for a command event
 * Returns :     void
 */
void CViewPlayer::OnPlay(wxCommandEvent& event)
{
	mPlay->Show(false);
	mPause->Show(true);
	if (mPlaying)
	{
		mTimer.Start();
		mTrueTime.Resume();
	}
	else
	{
		// Will attempt a refresh to achieve desired frame rate
		mTimer.Start(1000 / mDocument->GetTimeline()->GetFrameRate());
		// Some machines will not be able to update quickly enough
		mTrueTime.Start();
		mPlaying = true;
	}
}

/*
 * Name :        CViewPlayer::OnPause
 * Description : This will pause the animation
 * Parameters :  The object for a command event
 * Returns :     void
 */
void CViewPlayer::OnPause(wxCommandEvent& event)
{
	mTimer.Stop();
	mTrueTime.Pause();
	mPause->Show(false);
	mPlay->Show(true);
}

/*
 * Name :        CViewPlayer::OnStop
 * Description : This will stop the animation and restart it
 * Parameters :  The object for a command event
 * Returns :     void
 */
void CViewPlayer::OnStop(wxCommandEvent& event)
{
	mTimer.Stop();
	mTrueTime.Pause();
	mUsed.Clear(&mUnUsed);
	mPause->Show(false);
	mPlay->Show(true);
	mPlaying = false;
	mDocument->SetAnimationTime(0);
}

/*
 * Name :        CViewPlayer::OnClose
 * Description : This will safely 'close' the player window
 * Parameters :  The object for a paint event
 * Returns :     void
 */
void CViewPlayer::Closer()
{
	mPlaying = false;
	mPause->Show(false);
	mPlay->Show(true);
	mTrueTime.Pause();
	mTimer.Stop();
	mUsed.Empty();
	mUnUsed.Empty();
	mViewEdit->TogActive();
	mViewTimeline->TogActive();
	mViewActors->TogActive();
	mDocument->SetAnimationTime(mTempTime);
	this->Show( FALSE );
}

/*
 * Name :        CViewPlayer::CreateMoreSnow
 * Description : Add more snow to used from unused
 * Parameters :  none
 * Returns :     void
 */
void CViewPlayer::CreateMoreSnow()
{
	// Rate created depends on framerate/emissions
	int count = rand() % 10 / ((mDocument->GetTimeline()->GetFrameRate()) / 30.0);
	while (count > 0 && mUnUsed.Size() > count)
	{
		mUsed.AppendParticle(mUnUsed.GetParticle());
		count--;
	}
}

/*
 * Name :        CViewPlayer::InitializePool
 * Description : Creates new particles based on framerate
 * Parameters :  none
 * Returns :     none
 */
void CViewPlayer::InitializePool()
{
	// 5000 particles can support 10 per frame if 30fps
	for (int size = 0; size < 5000; size++)
	{
		CParticleSnowflake* snowflake = new CParticleSnowflake(mDocument->GetTimeline()->GetFrameRate());
		mUnUsed.AddParticle(snowflake);
	}
}
